@@ -92,6 +92,9 @@ class PaymentCommand(Command):
 
     # the followings are PaymentCommand specific methods
 
+    def my_subaddress(self, hrp: str) -> typing.Optional[bytes]:
+        return identifier.decode_account_subaddress(self.my_actor_address, hrp)
+
     def validate_state_trigger_actor(self) -> None:
         if self.inbound and self.opponent_actor() != self.state_trigger_actor():
             raise command_error(
